using System;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Configuration;

namespace DayOfDotNet
{
	public class OrderDetailDataGrid : System.Web.UI.Page
	{
		protected DataGrid OrderDataGrid;
		private DataSet ds = new DataSet();
		private string customerId;

		private void Page_Load(object sender, System.EventArgs e)
		{
			customerId = HttpContext.Current.Request.QueryString["customerid"].ToString();
			string sqlStmt = 
				"SELECT * FROM Orders WHERE CustomerID = @customerId; " + 
				"SELECT " +
				"[Orders].CustomerID, " +
				"[Order Details].OrderID, " +
				"[Order Details].ProductID, " + 
				"[Products].ProductName, " + 
				"[Categories].CategoryName, " + 
				"[Order Details].UnitPrice, " + 
				"[Order Details].Quantity, " + 
				"[Order Details].Discount, " + 
				"[Order Details].UnitPrice * " +
				"(1 - [Order Details].Discount) * " + 
				"[Order Details].Quantity AS SubTotal " +
				"FROM " +
				"[Order Details] " +
				"INNER JOIN " + 
				"[Products] ON [Order Details].ProductID = Products.ProductID " +
				"INNER JOIN " +
				"[Categories] ON Products.CategoryID = Categories.CategoryID " +
				"INNER JOIN "+
				"[Orders] ON [Order Details].OrderID = [Orders].OrderID " +
				"WHERE " +
				"[Orders].CustomerID = @customerId";
			string conString = ConfigurationSettings.AppSettings["NWDConString"];
			
			SqlDataAdapter sda = new SqlDataAdapter(sqlStmt, conString);
			sda.SelectCommand.Parameters.Add(
				new SqlParameter("@customerId", SqlDbType.NChar, 5)
				).Value = customerId;

			sda.Fill(ds);
			ds.Tables[0].TableName = "Orders";
			ds.Tables[1].TableName = "OrderDetail";
			
			Trace.Warn("Orders: " + ds.Tables["Orders"].Rows.Count.ToString());
			Trace.Warn("OrderDetail: " + ds.Tables["OrderDetail"].Rows.Count.ToString());
			
			OrderDataGrid.DataSource = ds.Tables["Orders"];
			OrderDataGrid.DataBind();
		}

		//Use the OnItemCreated event handler to dynamically add an embedded DataGrid
		protected void OrderDataGrid_OnItemDataBound(object sender, DataGridItemEventArgs e)
		{
			//When each row is created in the DataGrid, eval the ItemType
			if(e.Item.ItemType == ListItemType.Item || e.Item.ItemType == ListItemType.AlternatingItem)
			{
				//If the ItemType is Item or AlternatingItem, 
				//Create a new DataGrid object named DetailDataGrid
				DataGrid DetailDataGrid = new DataGrid();
				
				//Format the DataGrid to look cool.
				//***General Formatting***//
				DetailDataGrid.BorderWidth = (Unit)1;
				DetailDataGrid.Width = (Unit)640;
				DetailDataGrid.CellPadding = 4;
				DetailDataGrid.CellSpacing = 0;
				DetailDataGrid.GridLines = GridLines.Horizontal;
				DetailDataGrid.BorderColor = Color.FromName("Black");
				//***ItemStyle Formatting***//
				DetailDataGrid.ItemStyle.Font.Name = "Verdana";
				DetailDataGrid.ItemStyle.Font.Size = FontUnit.XSmall;
				//***AlternatingItemStyle Formatting***//
				DetailDataGrid.AlternatingItemStyle.BackColor = Color.FromName("LightGray");
				//***HeaderStyle Formatting***//
				DetailDataGrid.ShowHeader = true;
				DetailDataGrid.HeaderStyle.BackColor = Color.FromName("Black");
				DetailDataGrid.HeaderStyle.ForeColor = Color.FromName("White");
				DetailDataGrid.HeaderStyle.Font.Bold = true;
				DetailDataGrid.HeaderStyle.Font.Size = FontUnit.XSmall;

				
				//Do not autogenerate columns.
				DetailDataGrid.AutoGenerateColumns = false;
				
				//****Add a series of BoundColumns****//
				BoundColumn bc = new BoundColumn();				
				//Set the BoundColumn Values
				bc.HeaderText = "ID";
				bc.DataField = "ProductID";
				bc.ItemStyle.Wrap = false;
				//Add the BoundColumn to the DetailDataGrid.
				DetailDataGrid.Columns.Add(bc);

				//Do the same for the other columns
				//***Product Name***//
				bc = new BoundColumn();
				bc.HeaderText = "Product";
				bc.DataField = "ProductName";
				bc.ItemStyle.Wrap = true;
				DetailDataGrid.Columns.Add(bc);

				//***Category Name***//
				bc = new BoundColumn();
				bc.HeaderText = "Category";
				bc.DataField = "CategoryName";
				bc.ItemStyle.Wrap = false;
				DetailDataGrid.Columns.Add(bc);

				//***Unit Price***//
				bc = new BoundColumn();
				bc.HeaderText = "Unit Price";
				bc.DataField = "UnitPrice";
				bc.DataFormatString="{0:c}";
				bc.ItemStyle.Wrap = false;
				DetailDataGrid.Columns.Add(bc);

				//***Quantity***//
				bc = new BoundColumn();
				bc.HeaderText = "Qnt";
				bc.DataField = "Quantity";
				bc.ItemStyle.Wrap = false;
				DetailDataGrid.Columns.Add(bc);
				
				//***Discount***//
				bc = new BoundColumn();
				bc.HeaderText = "Dsc";
				bc.DataField = "Discount";
				bc.DataFormatString="{0:p}";
				bc.ItemStyle.Wrap = false;
				DetailDataGrid.Columns.Add(bc);

				//***SubTotal***//
				bc = new BoundColumn();
				bc.HeaderText = "SubTotal";
				bc.DataField = "SubTotal";
				bc.DataFormatString="{0:c}";
				bc.ItemStyle.Wrap = false;
				DetailDataGrid.Columns.Add(bc);
				
				//****End BoundColumns****//
				
				
				//Get the OrderDetail DataView and filter it 
				//for the current OrderID
				DataView _detail = ds.Tables["OrderDetail"].DefaultView;
				_detail.RowFilter = "OrderID='" + e.Item.Cells[0].Text + "'";
				
				//Bind the DataGrid.
				DetailDataGrid.DataSource = _detail;
				DetailDataGrid.DataBind();
				
				//Add the DetailDataGrid to the OrderDataGrid
				//in the 2nd column of the current row
				e.Item.Cells[1].Controls.Add(DetailDataGrid);
			}
		}
	}
}