using System;
using System.Data;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace DayOfDotNet
{
	public class SortingDataGrid2 : Page
	{
		protected DataGrid DataGrid1;

		private void Page_Load(object sender, System.EventArgs e)
		{
			if(!Page.IsPostBack)
			{
				//On the first request, Bind the DataGrid
				BindData();
			}
		}

		private void BindData()
		{
			//Create the DataView
			BooksData _booksData = new BooksData();
			DataView _data = _booksData.GetAllBooks();

			//If the ViewState SortExpression key has a value, 
			//sort the DataView, otherwise just bind the DataGrid
			if(ViewState["SortExpression"] != null) 
			{
				_data.Sort = ViewState["SortExpression"].ToString();
			}
			
			//Bind the DataGrid
			DataGrid1.DataSource = _data;
			DataGrid1.DataBind();
		}
		
		protected void SortCommand_OnClick(object sender, DataGridSortCommandEventArgs e)
		{
			//When a sort link is clicked, bind the 
			//DataGrid using the SortExpression
			//Since we will have alternating sorting, 
			//invoke the GetSortExpression() method
			//to put the sort expression with the
			//ASC or DESC attribute appended on it 
			//into ViewState
			GetSortExpression(e.SortExpression);
			BindData();
		}

		private void GetSortExpression(string _sortExpression)
		{
			//If the sort expression has not been put in
			//ViewState yet, do it now and return the
			//sort expression appended with ASC
			
			if(ViewState["SortExpression"] == null)
			{
				ViewState["SortExpression"] = _sortExpression + " ASC";
			}
			//If the sort expression is the same column the user clicked on
			//Check the sort direction
			else if((ViewState["SortExpression"].ToString()).StartsWith(_sortExpression))
			{
				//If the sort direction is ASC, make it DESC
				//If it is DESC make it ASC
				if((ViewState["SortExpression"].ToString()).EndsWith("ASC"))
				{
					ViewState["SortExpression"] =  _sortExpression + " DESC";
				}
				else
				{
					ViewState["SortExpression"] =  _sortExpression + " ASC";
				}
			}
			//If the sort expression is a different column that the last
			//sort expression, set the new sort expression in ViewState
			//and make its direction ASC
			else
			{
				ViewState["SortExpression"] = _sortExpression + " ASC";
			}
		}
	}
}