namespace PowerOfDotNet.admin
{
	using System;
	using System.Data;
	using System.Data.SqlClient;
	using System.Drawing;
	using System.Web;
	using System.Web.UI.WebControls;
	using System.Web.UI.HtmlControls;

	/// <summary>
	///		ModuleSettings   Դϴ.
	/// </summary>
	public abstract class ModuleSettingsControl : PowerOfDotNet.PortalModuleControlEx
	{
		protected System.Web.UI.WebControls.TextBox moduleTitle;
		protected System.Web.UI.WebControls.TextBox cacheTime;
		protected System.Web.UI.WebControls.CheckBoxList authEditRoles;
		protected System.Web.UI.WebControls.CheckBox showMobile;
		protected System.Web.UI.WebControls.LinkButton ApplyButton;

		int moduleId = 0;
		int tabId = 0;

		//*******************************************************
		//
		// The Page_Load server event handler on this page is used
		// to populate the module settings on the page
		//
		//*******************************************************

		private void Page_Load(object sender, System.EventArgs e) 
		{

			// Verify that the current user has access to access this page
			if (PortalSecurity.IsInRoles("Admins") == false) 
			{
				Response.Redirect("~/Admin/EditAccessDenied.aspx");
			}

			// Determine Module to Edit
			if (Request.Params["mid"] != null) 
			{
				moduleId = Int32.Parse(Request.Params["mid"]);
			}
			// Determine Tab to Edit
			if (Request.Params["tabid"] != null) 
			{
				tabId = Int32.Parse(Request.Params["tabid"]);
			}

			if (Page.IsPostBack == false) 
			{
				BindData();
			}
		}

		//*******************************************************
		//
		// The ApplyChanges_Click server event handler on this page is used
		// to save the module settings into the portal configuration system
		//
		//*******************************************************

		private void ApplyChanges_Click(Object Sender, EventArgs e) 
		{
    
			// Obtain PortalSettings from Current Context
			PortalSettings portalSettings = (PortalSettings) HttpContext.Current.Items["PortalSettings"];

			AdminDB admin = new AdminDB();
			SqlDataReader drModule = admin.GetModule(moduleId);
			drModule.Read();

			// Construct Authorized User Roles String
			String editRoles = "";

			foreach(ListItem item in authEditRoles.Items) 
			{

				if (item.Selected == true) 
				{
					editRoles = editRoles + item.Text + ";";
				}
			}
            
			int moduleOrder = Convert.ToInt32(drModule["ModuleOrder"]);
			string paneName = drModule["PaneName"].ToString();

			// update module
			admin.UpdateModule(moduleId, moduleOrder, paneName, moduleTitle.Text, Int32.Parse(cacheTime.Text), editRoles, showMobile.Checked);

			// Update Textbox Settings
			moduleTitle.Text = drModule["ModuleTitle"].ToString();
			cacheTime.Text = drModule["CacheTime"].ToString();

			// Populate checkbox list with all security roles for this portal
			// and "check" the ones already configured for this module
			SqlDataReader roles = admin.GetPortalRoles(portalSettings.PortalId);

			// Clear existing items in checkboxlist
			authEditRoles.Items.Clear();

			ListItem allItem = new ListItem();
			allItem.Text = "All Users";

			if (drModule["AuthorizedEditRoles"].ToString().LastIndexOf("All Users") > -1) 
			{
				allItem.Selected = true;
			}

			authEditRoles.Items.Add(allItem);

			while(roles.Read()) 
			{

				ListItem item = new ListItem();
				item.Text = (String) roles["RoleName"];
				item.Value = roles["RoleID"].ToString();

				if ((drModule["AuthorizedEditRoles"].ToString().LastIndexOf(item.Text)) > -1) 
				{
					item.Selected = true;
				}

				authEditRoles.Items.Add(item);
			}

			// Navigate back to admin page
			Response.Redirect(GetLink("-") + "&tabid=" + tabId);
		}

		//*******************************************************
		//
		// The BindData helper method is used to populate a asp:datalist
		// server control with the current "edit access" permissions
		// set within the portal configuration system
		//
		//*******************************************************

		private void BindData() 
		{
			// Obtain PortalSettings from Current Context
			PortalSettings portalSettings = (PortalSettings) HttpContext.Current.Items["PortalSettings"];

			AdminDB admin = new AdminDB();
			SqlDataReader drModule = admin.GetModule(moduleId);
			drModule.Read();

			moduleTitle.Text = drModule["ModuleTitle"].ToString();
			cacheTime.Text = drModule["CacheTime"].ToString();
			showMobile.Checked = Convert.ToBoolean(drModule["ShowMobile"]);

			// Populate checkbox list with all security roles for this portal
			// and "check" the ones already configured for this module
			SqlDataReader roles = admin.GetPortalRoles(portalSettings.PortalId);

			// Clear existing items in checkboxlist
			authEditRoles.Items.Clear();

			ListItem allItem = new ListItem();
			allItem.Text = "All Users";

			string moduleAuthorizedRoles = drModule["AuthorizedEditRoles"].ToString();
			string[] aModuleRoles = moduleAuthorizedRoles .Split(new char[] {';'});
			if (Array.IndexOf(aModuleRoles, "All Users") > -1)
			{
				allItem.Selected = true;
			}

			authEditRoles.Items.Add(allItem);

			while(roles.Read()) 
			{
				ListItem item = new ListItem();
				item.Text = (String) roles["RoleName"];
				item.Value = roles["RoleID"].ToString();

				if (Array.IndexOf(aModuleRoles,item.Text) > -1) 
				{
					item.Selected = true;
				}

				authEditRoles.Items.Add(item);
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN:  ȣ ASP.NET Web Form ̳ʿ ʿմϴ.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		̳  ʿ ޼Դϴ.
		///		 ޼  ڵ   ʽÿ.
		/// </summary>
		private void InitializeComponent()
		{
			this.ApplyButton.Click += new System.EventHandler(this.ApplyChanges_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion


	}
}
