using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections;

namespace CustomControls.TemplatedControls
{
	[ParseChildren(true)]
	public class TemplatedControl5 : WebControl, INamingContainer
	{
		private ITemplate itemTemplate = null;
		private ITemplate alternatingTemplate = null;
		private IEnumerable dataSource = null;

		public IEnumerable DataSource {
			get {return dataSource;}
			set {dataSource = value;}
		}

		[PersistenceMode(PersistenceMode.InnerProperty),
		TemplateContainer(typeof(TemplateItem))]
		public ITemplate ItemTemplate {
			get {return itemTemplate;}
			set {itemTemplate = value;}
		}

		[PersistenceMode(PersistenceMode.InnerProperty),
		TemplateContainer(typeof(TemplateItem))]
		public ITemplate AlternatingTemplate {
			get {return alternatingTemplate;}
			set {alternatingTemplate = value;}
		}

		private int ItemCount {
			get {
				object o = ViewState["itemCount"];
				return (o == null) ? 0 : (int)o;
			}
			set {ViewState["itemCount"] = value;}
		}

		protected override void CreateChildControls()
		{
			Controls.Clear();

			if(ItemCount != 0)
				CreateControlHierarchy(false);
		}

		protected override void OnDataBinding(System.EventArgs e)
		{
			base.OnDataBinding(e);

			Controls.Clear();

			if (dataSource != null) {
				if(HasChildViewState)
					ClearChildViewState();

				CreateControlHierarchy(true);
				ChildControlsCreated = true;
			}
		}

		private void CreateControlHierarchy(bool useDataSource) 
		{
			IEnumerable datSrc = null;

			if (useDataSource) 
				datSrc = this.dataSource;
			else 
				datSrc = new DummyDataSource(ItemCount);

			if (datSrc != null) {
				int count = 0;
				foreach(object dataItem in datSrc) {
					TemplateItem item = new TemplateItem(dataItem);
					if (count % 2 == 0 || alternatingTemplate == null)
						ItemTemplate.InstantiateIn(item);
					else
						alternatingTemplate.InstantiateIn(item);

					Controls.Add(item);
					count++;
				}

				if (useDataSource) 
					ItemCount = count;
			}
		}

		public class TemplateItem : Control, INamingContainer 
		{
			private object dataItem;

			public TemplateItem(object datItm) 
			{
				dataItem = datItm;
			}

			public object DataItem {
				get {return dataItem;}
				set {dataItem = value;}
			}
		}

		internal class DummyDataSource : IEnumerable
		{
			private int itemCount;

			public DummyDataSource(int itemCount) {
				this.itemCount = itemCount;
			}

			public IEnumerator GetEnumerator() {
				return new DummyDataSourceEnumerator(itemCount);
			}

			private class DummyDataSourceEnumerator : IEnumerator 
			{
				private int count;
				private int index;

				public DummyDataSourceEnumerator(int count) {
					this.count = count;
					this.index = -1;
				}

				public object Current {
					get {return null;}
				}

				public bool MoveNext() {
					index++;
					return index < count;
				}

				public void Reset() {
					this.index = -1;
				}
			}
		}
	}
}
