/*
 *  - 2003 10 26
 */

using System;
using System.Configuration;
using System.Web;
using System.Data;
using System.Data.SqlClient;
using System.Collections;

using PowerOfDotNet.TypedDataSet;

namespace PowerOfDotNet {

    //*********************************************************************
    //
    // TabStripDetails Class
    //
    // Class that encapsulates the just the tabstrip details -- TabName, TabId and TabOrder 
    // -- for a specific Tab in the Portal
    //
    //*********************************************************************

    public class TabStripDetails {

        public int          TabId;
        public String       TabName;
        public int          TabOrder;
        public String       AuthorizedRoles;
    }

    //*********************************************************************
    //
    // PageSettings Class
    //
    // Class that encapsulates the detailed settings for a specific Page 
    // in the Portal
    //
    //*********************************************************************

    public class PageSettings {
                           
        public int          TabIndex;
        public int          TabId;
		public int			PageID;
        public String       TabName;
        public int          TabOrder;
        public String       MobileTabName;
        public String       AuthorizedRoles;
		public String		PageTemplateName;
        public bool         ShowMobile;
        public ArrayList    Modules = new ArrayList();
    }

    //*********************************************************************
    //
    // ModuleSettings Class
    //
    // Class that encapsulates the detailed settings for a specific Tab 
    // in the Portal
    //
    //*********************************************************************

    public class ModuleSettings {

        public int          ModuleId;
        public int          ModuleDefId;
        public int          TabId;
		public int			PageID;
        public int          CacheTime;
		//public CacheModeType	CacheMode;
		public string		CacheDependencyString;
        public int          ModuleOrder;
        public String       PaneName;
        public String       ModuleTitle;
        public String       AuthorizedEditRoles;
        public bool         ShowMobile;
        public String       DesktopSrc;
        public String       MobileSrc;
    }

    //*********************************************************************
    //
    // PortalSettings Class
    //
    // This class encapsulates all of the settings for the Portal, as well
    // as the configuration settings required to execute the current tab
    // view within the portal.
    //
    //*********************************************************************

    public class PortalSettings {

        public int          PortalId;
        public String       PortalName;
        public bool         AlwaysShowEditButton;
        public ArrayList    DesktopTabs = new ArrayList();
        public ArrayList    MobileTabs = new ArrayList();
		public int			CacheTimeout;
        public PageSettings  ActivePage = new PageSettings();

        //*********************************************************************
        //
        // PortalSettings Constructor
        //
        // The PortalSettings Constructor encapsulates all of the logic
        // necessary to obtain configuration settings necessary to render
        // a Portal Tab view for a given request.
        //
        // These Portal Settings are stored within a SQL database, and are
        // fetched below by calling the "GetPortalSettings" stored procedure.
        // This stored procedure returns values as SPROC output parameters,
        // and using three result sets.
        //
        //*********************************************************************
		// 2003 10 30 
		//   ε ˾Ƴ  ߰
		//   ĳÿ ϵ 

		public PortalSettings(int tabId, int pageid) {
			TypedDataSet.PortalSettingsDS portalSettingsData = 
				(TypedDataSet.PortalSettingsDS)
					HttpContext.Current.Cache["__PortalSettingsCacheData"];
			//bool useCache = false;
			int cacheTime;

			this.CacheTimeout = cacheTime = GetPortalCacheTime();

			if (cacheTime > 0) { //  ĳÿ 
				if (portalSettingsData == null)
					portalSettingsData = CreateCacheDB(cacheTime);

				PortalSettingsFromCache(tabId, pageid, portalSettingsData);
			}
			else {// û DB ׼
				PortalSettingsFromDB(tabId, pageid);
			}
        }


		//ӿũ ͸ ɽ÷ Ѵ.
		private TypedDataSet.PortalSettingsDS CreateCacheDB(int cacheTime) {
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);

			SqlDataAdapter daPortals = new SqlDataAdapter("GetAllPortals", myConnection);
			SqlDataAdapter daTabs = new SqlDataAdapter("GetAllTabs", myConnection);
			SqlDataAdapter daPages = new SqlDataAdapter("GetAllPages", myConnection);
			SqlDataAdapter daModuleDefinitions = new SqlDataAdapter("GetAllModuleDefinitions", myConnection);
			SqlDataAdapter daModules = new SqlDataAdapter("GetAllModules", myConnection);

			TypedDataSet.PortalSettingsDS tds = new TypedDataSet.PortalSettingsDS();

			daPortals.SelectCommand.CommandType = CommandType.StoredProcedure;
			daPortals.FillSchema(tds, SchemaType.Source, "Portals");
			daPortals.Fill(tds, "Portals");

			daTabs.SelectCommand.CommandType = CommandType.StoredProcedure;
			daTabs.FillSchema(tds, SchemaType.Source, "Tabs");
			daTabs.Fill(tds, "Tabs");

			daPages.SelectCommand.CommandType = CommandType.StoredProcedure;
			daPages.FillSchema(tds, SchemaType.Source, "Pages");
			daPages.Fill(tds, "Pages");

			daModuleDefinitions.SelectCommand.CommandType = CommandType.StoredProcedure;
			daModuleDefinitions.FillSchema(tds, SchemaType.Source, "ModuleDefinitions");
			daModuleDefinitions.Fill(tds, "ModuleDefinitions");

			daModules.SelectCommand.CommandType = CommandType.StoredProcedure;
			daModules.FillSchema(tds, SchemaType.Source, "Modules");
			daModules.Fill(tds, "Modules");

			//׼ Ƚ ()
			tds.ExtendedProperties.Add("accessCount", 0);

			HttpContext.Current.Cache.Insert("__PortalSettingsCacheData", tds, null, DateTime.Now.AddMinutes(cacheTime), TimeSpan.Zero);

			return tds;
		}

		private int GetPortalCacheTime()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("GetPortalCacheTime", myConnection);

			// Mark the Command as a SPROC
			myCommand.CommandType = CommandType.StoredProcedure;

			SqlParameter parameterPortalAlias = new SqlParameter("@PortalAlias", SqlDbType.NVarChar, 50);
			parameterPortalAlias.Value = "p_default";
			myCommand.Parameters.Add(parameterPortalAlias);

			myCommand.Parameters.Add("RETURN_VALUE", SqlDbType.Int).Direction = ParameterDirection.ReturnValue;

			myConnection.Open();

			int cacheTime = -1;
			try 
			{
				myCommand.ExecuteNonQuery();
				cacheTime = Convert.ToInt32(myCommand.Parameters["RETURN_VALUE"].Value);
			}
			catch (Exception err)
			{
				string s = err.Message;
			}
			finally
			{
				myConnection.Close();
			}

			return cacheTime;
		}
    

		private void PortalSettingsFromCache(int tabId, int pageid, PortalSettingsDS tds) {
			
			PortalSettingsDS.PortalsRow portalRow;
			PortalSettingsDS.TabsRow[] tabsRows;

			if (tabId == 0) 
			{
				portalRow = (PortalSettingsDS.PortalsRow)tds.Portals.Select("PortalAlias=" + "'p_default'")[0];
			}
			else 
			{
				PortalSettingsDS.TabsRow tabsRow = tds.Tabs.FindByTabID(tabId);
				portalRow = tds.Portals.FindByPortalID(tabsRow.PortalID);
			}

			tabsRows = portalRow.GetTabsRows();

			int tabIndex = -1;
			int tabCount = 0;
			//tabId 0 쿡 Ʒ κп tabIndex  Ƿ ʱȭ ش.
			if (tabId == 0)
				tabIndex = 0;
			foreach (PortalSettingsDS.TabsRow ltr in tabsRows) 
			{
				TabStripDetails tabDetails = new TabStripDetails();
				tabDetails.TabId = ltr.TabID;
				tabDetails.TabName = ltr.TabName;
				tabDetails.TabOrder = ltr.TabOrder;
				tabDetails.AuthorizedRoles = ltr.AuthorizedRoles;

				this.DesktopTabs.Add(tabDetails);

				if (ltr.ShowMobile) {
					TabStripDetails mTabDetails = new TabStripDetails();
					mTabDetails.TabId = ltr.TabID;
					mTabDetails.TabName = ltr.MobileTabName;
					mTabDetails.AuthorizedRoles = ltr.AuthorizedRoles;
				}

				//  ε ˱
				if (tabId == tabDetails.TabId)
					tabIndex = tabCount;
				tabCount++;
			}
			if (tabId == 0) 
			{
				tabId = ((TabStripDetails) this.DesktopTabs[0]).TabId;
			}
				
			PortalSettingsDS.ModulesRow[] modulesRows = 
				(PortalSettingsDS.ModulesRow[])tds.Modules.Select(
					"TabID = " + tabId + " and TargetID = " + pageid + 
					" and PortalID = " + portalRow.PortalID, "ModuleOrder");
			/*
			PortalSettingsDS.ModuleDefinitionsRow moduleDefinitionsRow = 
				(PortalSettingsDS.ModuleDefinitionsRow)modulesRows[0].GetParentRow(
					tds.Relations["ModuleDefinitionsModules"]);
					*/

			foreach(PortalSettingsDS.ModulesRow lmr in modulesRows) {
				ModuleSettings m = new ModuleSettings();
				m.ModuleId = lmr.ModuleID;
				m.ModuleDefId = lmr.ModuleDefID;
				m.TabId = lmr.TabID;
				m.PageID = lmr.TargetID;
				m.PaneName = lmr.PaneName;
				m.ModuleTitle =	lmr.ModuleTitle;
				m.AuthorizedEditRoles = lmr.AuthorizedEditRoles;
				m.CacheTime = lmr.CacheTime;

				m.CacheDependencyString = lmr.CacheDependencyString;
				m.ModuleOrder = lmr.ModuleOrder;
				m.ShowMobile = lmr.ShowMobile;

				PortalSettingsDS.ModuleDefinitionsRow moduleDefinitionsRow = 
					(PortalSettingsDS.ModuleDefinitionsRow)
						lmr.GetParentRow(tds.Relations["ModuleDefinitionsModules"]);
				m.DesktopSrc = moduleDefinitionsRow.DesktopSrc;
				m.MobileSrc = moduleDefinitionsRow.MobileSrc;

				this.ActivePage.Modules.Add(m);
			}

			PortalSettingsDS.PagesRow pagesRow = 
				(PortalSettingsDS.PagesRow)tds.Pages.Select(
					"PortalID = " + portalRow.PortalID + " and TabID=" + 
					tabId + " and TargetID=" + pageid)[0];

			this.ActivePage.TabId = tabsRows[tabIndex].TabID;
			this.ActivePage.TabOrder = tabsRows[tabIndex].TabOrder;
			this.PortalId = portalRow.PortalID;
			this.PortalName = portalRow.PortalName;
			this.AlwaysShowEditButton = portalRow.AlwaysShowEditButton;
			this.ActivePage.PageID = pageid;
			this.ActivePage.TabIndex = tabIndex;
			this.ActivePage.MobileTabName = tabsRows[tabIndex].MobileTabName;
			this.ActivePage.PageTemplateName = pagesRow.PageTemplate;
			this.ActivePage.AuthorizedRoles = pagesRow.AuthorizedRoles;
			this.ActivePage.TabName = tabsRows[tabIndex].TabName;
			this.ActivePage.ShowMobile = tabsRows[tabIndex].ShowMobile;
		}

		private void PortalSettingsFromDB(int tabId, int pageid) {

			// Create Instance of Connection and Command Object
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("GetPortalSettings", myConnection);

			// Mark the Command as a SPROC
			myCommand.CommandType = CommandType.StoredProcedure;

			// Add Parameters to SPROC
			SqlParameter parameterPortalAlias = new SqlParameter("@PortalAlias", SqlDbType.NVarChar, 50);
			parameterPortalAlias.Value = "p_default";
			myCommand.Parameters.Add(parameterPortalAlias);

			SqlParameter parameterTabId = new SqlParameter("@TabId", SqlDbType.Int, 4);
			parameterTabId.Value = tabId;
			myCommand.Parameters.Add(parameterTabId);

			SqlParameter parameterPageid = new SqlParameter("@PageID", SqlDbType.Int, 4);
			parameterPageid.Value = pageid;
			myCommand.Parameters.Add(parameterPageid);

			// Add out parameters to Sproc
			SqlParameter parameterPortalID = new SqlParameter("@PortalID", SqlDbType.Int, 4);
			parameterPortalID.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterPortalID);

			SqlParameter parameterPortalName = new SqlParameter("@PortalName", SqlDbType.NVarChar, 128);
			parameterPortalName.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterPortalName);

			SqlParameter parameterEditButton = new SqlParameter("@AlwaysShowEditButton", SqlDbType.Bit, 1);
			parameterEditButton.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterEditButton);

			SqlParameter parameterTabName = new SqlParameter("@TabName", SqlDbType.NVarChar, 50);
			parameterTabName.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterTabName);

			SqlParameter parameterTabOrder = new SqlParameter("@TabOrder", SqlDbType.Int, 4);
			parameterTabOrder.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterTabOrder);

			SqlParameter parameterPageTemplate = new SqlParameter("@PageTemplate", SqlDbType.VarChar, 100);
			parameterPageTemplate.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterPageTemplate);

			SqlParameter parameterMobileTabName = new SqlParameter("@MobileTabName", SqlDbType.NVarChar, 50);
			parameterMobileTabName.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterMobileTabName);

			SqlParameter parameterAuthRoles = new SqlParameter("@AuthRoles", SqlDbType.NVarChar, 256);
			parameterAuthRoles.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterAuthRoles);

			SqlParameter parameterShowMobile = new SqlParameter("@ShowMobile", SqlDbType.Bit, 1);
			parameterShowMobile.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterShowMobile);

			/*
			SqlParameter parameterCacheTime = new SqlParameter("@CacheTime", SqlDbType.Int, 4);
			parameterCacheTime.Direction = ParameterDirection.Output;
			myCommand.Parameters.Add(parameterCacheTime);
			*/

			// Open the database connection and execute the command
			myConnection.Open();
			SqlDataReader result = myCommand.ExecuteReader();

			// Read the first resultset -- Desktop Tab Information
			int tabIndex = -1;
			int tabCount = 0;
			//tabId 0 쿡 Ʒ κп tabIndex  Ƿ ʱȭ ش.
			if (tabId == 0)
				tabIndex = 0;
			while(result.Read()) 
			{

				TabStripDetails tabDetails = new TabStripDetails();
				tabDetails.TabId = (int) result["TabId"];
				tabDetails.TabName = (String) result["TabName"];
				tabDetails.TabOrder = (int) result["TabOrder"];
				tabDetails.AuthorizedRoles = (String) result["AuthorizedRoles"];

				this.DesktopTabs.Add(tabDetails);
				//  ε ˱
				if (tabId == tabDetails.TabId)
					tabIndex = tabCount;
				tabCount++;
			}

			if (this.ActivePage.TabId == 0) 
			{
				this.ActivePage.TabId = ((TabStripDetails) this.DesktopTabs[0]).TabId; 
			}

			// Read the second result --  Mobile Tab Information
			result.NextResult();

			while(result.Read()) 
			{

				TabStripDetails tabDetails = new TabStripDetails();
				tabDetails.TabId = (int) result["TabId"];
				tabDetails.TabName = (String) result["MobileTabName"];
				tabDetails.AuthorizedRoles = (String) result["AuthorizedRoles"];

				this.MobileTabs.Add(tabDetails);
			}

			// Read the third result --  Module Tab Information
			result.NextResult();

			while(result.Read()) 
			{

				ModuleSettings m = new ModuleSettings();
				m.ModuleId = (int) result["ModuleID"];
				m.ModuleDefId = (int) result["ModuleDefID"];
				m.TabId = (int) result["TabID"];
				m.PageID = (int) result["TargetID"];
				m.PaneName = (String) result["PaneName"];
				m.ModuleTitle = (String) result["ModuleTitle"];
				m.AuthorizedEditRoles = (String) result["AuthorizedEditRoles"];
				m.CacheTime = (int) result["CacheTime"];
				/*
				switch ((System.Byte) result["CacheMode"])
				{
					case 0 : 
						m.CacheMode = CacheModeType.TimeOnly;
						break;
					case 1 : 
						m.CacheMode = CacheModeType.DatabaseLocally;
						break;
					case 2 : 
						m.CacheMode = CacheModeType.DatabaseRemotely;
						break;
				}
				*/

				m.CacheDependencyString = (string) result["CacheDependencyString"];
				m.ModuleOrder = (int) result["ModuleOrder"];
				m.ShowMobile = (bool) result["ShowMobile"];
				m.DesktopSrc = (String) result["DesktopSrc"];
				m.MobileSrc = (String) result["MobileSrc"];

				this.ActivePage.Modules.Add(m);
			}

			// Now read Portal out params 
			result.NextResult();

			this.PortalId = (int) parameterPortalID.Value;
			this.PortalName = (String) parameterPortalName.Value;
			this.AlwaysShowEditButton = (bool) parameterEditButton.Value;
			this.ActivePage.TabIndex = tabIndex;
			//this.ActivePage.TabIndex = (int) parameterTabOrder.Value;
			this.ActivePage.TabId = tabId;
			this.ActivePage.PageID = pageid;
			this.ActivePage.TabOrder = (int) parameterTabOrder.Value;
			this.ActivePage.MobileTabName = (String) parameterMobileTabName.Value;
			this.ActivePage.PageTemplateName = (string) parameterPageTemplate.Value;
			this.ActivePage.AuthorizedRoles = (String) parameterAuthRoles.Value;
			this.ActivePage.TabName = (String) parameterTabName.Value;
			this.ActivePage.ShowMobile = (bool) parameterShowMobile.Value;

			myConnection.Close();

			//ĳ ȯ
			//return (int)parameterCacheTime.Value > 0;
		}

        //*********************************************************************
        //
        // GetModuleSettings Static Method
        //
        // The PortalSettings.GetModuleSettings Method returns a hashtable of
        // custom module specific settings from the database.  This method is
        // used by some user control modules (Xml, Image, etc) to access misc
        // settings.
        //
        //*********************************************************************

        public static Hashtable GetModuleSettings(int moduleId) {

            // Get Settings for this module from the database
            Hashtable _settings = new Hashtable();

            // Create Instance of Connection and Command Object
            SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
            SqlCommand myCommand = new SqlCommand("GetModuleSettings", myConnection);

            // Mark the Command as a SPROC
            myCommand.CommandType = CommandType.StoredProcedure;

            // Add Parameters to SPROC
            SqlParameter parameterModuleId = new SqlParameter("@ModuleID", SqlDbType.Int, 4);
            parameterModuleId.Value = moduleId;
            myCommand.Parameters.Add(parameterModuleId);

            // Execute the command
            myConnection.Open();
            SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

            while (dr.Read()) {

                _settings[dr.GetString(0)] = dr.GetString(1);
            }

            dr.Close();

            return _settings;
        }
    }
}