// Copyright (C) 2002 Mark Pasternak
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
// Mark Pasternak (mark.pasternak@universum.se)


// Ǿ ASP.NET ڷǿ ۿ  ٿε ŬԴϴ.
// ûϽ  ż  ּ ޾ҽϴ.
//
// ּ ÷ :  (http://powerof.net)


using System;
using System.Web;
using System.IO;

namespace MarkPasternak.Utility
{
	/// <summary>
	/// The HttpResponse.WriteFile method does not work very well with large files since it reads
	/// the whole file to memory at once. This may crash the ASP.NET worker process if the file is too big.
	/// This helper class uses buffers and continuously checks if the client is connected before it sends any output.
	///
	/// HttpResponse.WriteFile ޼ҵ  ū  ϴ    ʽϴ. 
	/// ֳĸ  ޼ҵ   ޸𸮷 ѹ  о帮  
	///   ũٸ   δ ݴϴ. ɰ  ASP.NET μ ״ 쵵 .
	///  Ŭ ۸ ̿  а ϱ  
	///  HttpResponse.WriteFile ޼ҵ  ذݴϴ.
	/// ׸ Ŭ̾Ʈ   ϱ⶧ Ŭ̾Ʈ  
	/// ϰų ٸ    ̻   ʽϴ.
	/// </summary>
	public class WriteFileHelper
	{
		private int m_bufferSize=4096; //ũ⸦ 4ް 
		private HttpContext Context;
		public EventHandler DownloadCancelled;
		public EventHandler DownloadCompleted;

		public WriteFileHelper()
		{
			Context = HttpContext.Current;
		}

		/// <summary>
		/// Sets and gets the size of the buffer that is used when a file is read to memory.
		/// A larger buffer will require more memory, but will on the other hand make 
		/// it less resource intensive to send a file. Experiment to find a good balance.
		/// </summary>
		///  m_bufferSize 4096 ⺻ 
		/// Ѵٸ  Ӽ ̿ ũ⸦ Ҽ ִ.
		public int BufferSize
		{
			set{m_bufferSize=value;}
			get{return m_bufferSize;}
		}

		//    ϰų   Ŭ̾Ʈ   
		//  ޼ҵ尡 ȣ˴ϴ.
		//  Ŭ ܺο DownloadCancelled  ̺Ʈ 鷯 ϸ
		//  ޼ҵ尡 ȣɶ  ó Ҽ ֽϴ.
		protected void OnDownloadCancelled() 
		{ 
			if (DownloadCancelled != null) // 鷯  ִٸ
				DownloadCancelled(null, null); 
		}

		//  ϷǸ  ޼ҵ尡 ȣ˴ϴ.
		//  Ŭ ܺο DownloadCompleted  ̺Ʈ 鷯 ϸ
		//  ޼ҵ尡 ȣɶ  ó Ҽ ֽϴ.
		protected void OnDownloadCompleted() 
		{ 
			if (DownloadCompleted != null) // 鷯  ִٸ
				DownloadCompleted(null, null); 
		}

		/// <summary>
		/// Writes a file to the Response Stream
		/// </summary>
		/// <param name="filePath">A Path to a file</param>
		///   θ ڷ  ޼ҵ带 ȣϸ 
		/// Ŭ̾Ʈ  ϴ.
		/// ׷  ޼ҵ   
		/// WriteFileToResponseStreamWithForceDownloadHeaders ޼ҵ带 ϴ°  մϴ.
		/// ׷   ޼ҵ     ϱ
		public void WriteFileToResponseStream(string filePath)
		{
			if(!File.Exists(filePath)) // û    Ų.
				throw new Exception("File Path does not exist : " + filePath);

			// Ŭ̾Ʈ  Ѵ.
			WriteBinaryFile_Internal(filePath);
		}

		/// <summary>
		/// Writes a file to the Response Stream and adds the headers sthe "Save As" dialog is presented to the user
		/// </summary>
		/// <param name="filePath">A Path to a file</param>
		///   θ ڷ  ޼ҵ带 ȣϸ 
		/// Ŭ̾Ʈ HTTP  ÷ؼ  մϴ.
		/// ÷   Ŭ̾Ʈ '','', '' ư ִ 
		/// ȭڰ ϴ.
		public void WriteFileToResponseStreamWithForceDownloadHeaders(string filePath)
		{
			if(!File.Exists(filePath)) // û    Ų.
				throw new Exception("File Path does not exist : " + filePath);

			Context.Response.Clear();
			// ش ߰Ѵ
			Context.Response.ContentType = "application/octet-stream";
			Context.Response.AddHeader("Content-Disposition", "attachment; filename=" + Path.GetFileName(filePath));
			Context.Response.AddHeader("Content-Length", new FileInfo(filePath).Length.ToString());

			// Ŭ̾Ʈ  Ѵ.
			WriteBinaryFile_Internal(filePath);
		}

		//   ϴ ޼ҵ
		private void WriteBinaryFile_Internal(string filePath)
		{
			Context.Response.Buffer=false; // ۸  ʴ´.
			FileStream inStr = null;
			byte[] buffer = new byte[m_bufferSize]; //  о帱 
			long byteCount;
			try
			{
				inStr = File.OpenRead(filePath); //  .
				// m_bufferSize ũ ŭ   ʴ оδ.
				while ((byteCount = inStr.Read(buffer, 0, buffer.Length)) > 0) 
				{
					// Ŭ̾Ʈ  ִ ˻Ѵ.
					// , ڰ  ߰ ϰų
					// ٸ    IsClientConnected talse ȴ.
					if(Context.Response.IsClientConnected)
					{
						// ۿ  ͸ Ŭ̾Ʈ Ѵ.
						Context.Response.OutputStream.Write(buffer, 0, buffer.Length);
						Context.Response.Flush();
					}
					else 
					{
						//    ˸ ̺Ʈ ߻Ų.
						OnDownloadCancelled();
						return;
					}
				}
				OnDownloadCompleted(); //  Ϸ ̺Ʈ ߻Ų.
			}
			catch(Exception ex)
			{
				throw ex;
			}
			finally
			{
				inStr.Close();
				Context.Response.End();
			}
		}


	}
}
